# NaifJs, a simple state-machine based dialog manager

NaifJs is a simple state-machine based (task-oriented) 
[dialog manager](https://en.wikipedia.org/wiki/Dialog_manager) micro-framework for nodejs developers.

The package consists in: 
- A run-time dialog engine as a library to be embedded in production conversational apps 
- A micro-framework to create and test dialogues in dev environment (`naif` command line program)


## [Concepts](concepts.md)

Naifjs is based on a specific conceptual model and a domain specific language (DSL) I conceived,
implementing multi-turn contextual dialogues as "state machines". 

Following paragraph introduce main concepts and naming conventions:

- Dialog system
  - [Stateful multi-turn dialogues](doc/concepts.md#introduction--stateful-multi-turn-dialogues)
  - [Architecture](doc/concepts.md#architecture)

- State-machine based dialog manager 
  - [Dialogue as a state-machine](doc/concepts.md#dialogue-as-a-state-machine)
  - [State tracker](doc/concepts.md#state-tracker)

- What is a dialog unit
  - [Dialog Unit](doc/concepts.md#dialog-unit)
  - [Coding a dialogue with a javascript domain specific language](doc/concepts.md#coding-a-dialogue-with-a-javascript-domain-specific-language)
  - [Requests and responses](doc/concepts.md#requests-and-responses)

- Application 
  - [Dialog application](doc/concepts.md#dialog-application)
  - [Sessions](doc/sessions.md)


## Installation

The package contains command line interface program `naif`, so you must install the npm package as global:

- use npm package manager repo

```
$ npm install -g naifjs
```

- or download this github repo:

```
$ git clone https://github.com/solyarisoftware/naifjs
$ cd naifjs && npm link
``` 

## API and tools  

### [API library](doc/API.md)

Currently NaifJs act as an API library of functions to be called by a main nodejs program.
See [API](doc/API.md) documentations and examples. 

### [`naif` command line program](doc/API.md)

The CLI program [`naif`](doc/CLI.md) acts as a command line tester/debugger, 
a dialog code generator and more.

- [`naif show`](doc/CLI.md#naif-show) - Application dialogues directory info
- [`naif test shell`](doc/CLI.md#naif-test) /[`naif test telegram`](doc/CLI.md#naif-telegram) - How to test dialogues on the command line / with a Telegram bot
- [`naif new dialog`](doc/CLI.md#naif-new) - A basic dialogue code skeleton generator


## Development environment workflow

👉 A discussion about the design/development [workflow](doc/workflow.md).


## Discussion 

- [Pros and cons](doc/discussion.md#pros-and-cons)
- [Food for thought](doc/discussion.md#food-for-thought)
- [BackStory](doc/discussion.md#backstory)
- [Acknowledgments](doc/discussion.md#acknowledgments)


## How to contribute

Please feel free to contact me for any proposal/discussion. 
There are many areas of improvement!

- [how to contribute](doc/contributing.md).<br>
- [To do](doc/todo.md)


## Caveat

Currently the project is just a proof of concept. Not ready for production.
<br>The software made is just a prototype in alpha stage. 


## License 

[MIT](LICENSE.md) (c) Giorgio Robino

---

[top](#) | [index](doc/index.md)

